﻿using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System;
using System.Collections.Generic;
using System.Dynamic;
using System.Globalization;

namespace ITOps.Json;

internal class PascalCaseExpandoObjectConverter : ExpandoObjectConverter
{
    public override object ReadJson(JsonReader reader, Type objectType, object existingValue, JsonSerializer serializer) =>
        ReadValue(reader);

    private object ReadValue(JsonReader reader)
    {
        while (reader.TokenType == JsonToken.Comment)
        {
            if (!reader.Read())
            {
                throw new Exception("Unexpected end.");
            }
        }

        switch (reader.TokenType)
        {
            case JsonToken.StartObject:
                return ReadObject(reader);

            case JsonToken.StartArray:
                return ReadList(reader);

            default:
                if (IsPrimitiveToken(reader.TokenType))
                {
                    return reader.Value;
                }

                throw new Exception(string.Format(CultureInfo.InvariantCulture, "Unexpected token when converting ExpandoObject: {0}", reader.TokenType));
        }
    }

    private object ReadList(JsonReader reader)
    {
        var list = new List<object>();

        while (reader.Read())
        {
            switch (reader.TokenType)
            {
                case JsonToken.Comment:
                    break;

                default:
                    list.Add(ReadValue(reader));
                    break;

                case JsonToken.EndArray:
                    return list;
            }
        }

        throw new Exception("Unexpected end.");
    }

    private object ReadObject(JsonReader reader)
    {
        IDictionary<string, object> expandoObject = new ExpandoObject();

        while (reader.Read())
        {
            switch (reader.TokenType)
            {
                case JsonToken.PropertyName:
                    var propertyName = reader.Value.ToString().ToPascalCase();

                    if (!reader.Read())
                    {
                        throw new Exception("Unexpected end.");
                    }

                    expandoObject[propertyName] = ReadValue(reader);
                    break;

                case JsonToken.Comment:
                    break;

                case JsonToken.EndObject:
                    return expandoObject;
            }
        }

        throw new Exception("Unexpected end.");
    }

    private static bool IsPrimitiveToken(JsonToken token)
    {
        switch (token)
        {
            case JsonToken.Integer:
            case JsonToken.Float:
            case JsonToken.String:
            case JsonToken.Boolean:
            case JsonToken.Null:
            case JsonToken.Undefined:
            case JsonToken.Date:
            case JsonToken.Bytes:
                return true;

            default:
                return false;
        }
    }
}